"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.errorCatchHandler = exports.verifyOutput = exports.regexResult = exports.repoHasLfs = exports.repoHasLfsFilters = exports.loadGitattributeFiltersFromRepo = exports.getGitattributesPathFromRepo = void 0;

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _ramda = _interopRequireDefault(require("ramda"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getGitattributesPathFromRepo = repo => _path.default.join(repo.workdir(), '.gitattributes');

exports.getGitattributesPathFromRepo = getGitattributesPathFromRepo;

const loadGitattributeFiltersFromRepo = repo => {
  const gitattrPath = getGitattributesPathFromRepo(repo);
  return _fsExtra.default.pathExists(gitattrPath).then(exists => {
    if (!exists) {
      return '';
    }

    return _fsExtra.default.readFile(gitattrPath, 'utf8');
  }).then(fileContents => {
    const attributes = fileContents.split('\n');

    const lfsFilters = _ramda.default.reduce((acc, line) => {
      if (!_ramda.default.contains(_constants.LFS_ATTRIBUTE, line)) {
        return acc;
      }

      const start = line.indexOf(_constants.LFS_ATTRIBUTE);
      const filter = line.substring(0, start);
      acc.push(filter.trim());
      return acc;
    }, [], attributes);

    return lfsFilters;
  });
};

exports.loadGitattributeFiltersFromRepo = loadGitattributeFiltersFromRepo;

const repoHasLfsFilters = repo => loadGitattributeFiltersFromRepo(repo).then(filters => filters.length > 0).catch(() => false);

exports.repoHasLfsFilters = repoHasLfsFilters;
const repoHasLfs = repoHasLfsFilters;
exports.repoHasLfs = repoHasLfs;

const regexResult = (input, regularExpression) => input.match(regularExpression);

exports.regexResult = regexResult;

const verifyOutput = (stats, raw) => {
  // Check to see if it was a permissions error
  const wasPermissionDenied = raw.trim().toLowerCase().match(_constants.regex.PERMISSION_DENIED);

  if (wasPermissionDenied) {
    const e = new Error(wasPermissionDenied[0]);
    e.errno = _constants.BAD_CORE_RESPONSE;
    throw e;
  } // We need to handle this manually because LFS isn't returning stderr


  const props = _ramda.default.values(stats);

  const allErrored = _ramda.default.pipe(_ramda.default.filter(_ramda.default.equals(_constants.BAD_REGEX_PARSE_RESULT)), _ramda.default.sum, _ramda.default.equals(props.length))(props); // We have all errors


  if (allErrored) {
    const e = new Error(raw);
    e.errno = _constants.BAD_CORE_RESPONSE;
    throw e;
  }
};

exports.verifyOutput = verifyOutput;

const errorCatchHandler = response => err => {
  // This is a manually detected error we get from LFS
  if (err.errno === _constants.BAD_CORE_RESPONSE) {
    response.stderr = response.raw;
    response.errno = _constants.BAD_CORE_RESPONSE;
    response.success = false;
    return response;
  }

  throw err;
};

exports.errorCatchHandler = errorCatchHandler;