"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dependencyCheck = exports.isAtleastLfsVersion = exports.isAtleastGitVersion = exports.parseVersion = void 0;

var _ramda = _interopRequireDefault(require("ramda"));

var _generateResponse = _interopRequireDefault(require("./generateResponse"));

var _lfsCommands = require("../commands/lfsCommands");

var _constants = require("../constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @function normalizeVersion
 * @param  Array<string> versionArray array of version number eg: ['1', '8', '3'] => 1.8.3
 * @return Number normalized version number
 */
const normalizeVersion = versionArray => {
  if (!versionArray || versionArray.length === 0) {
    return _constants.BAD_VERSION;
  }

  return _ramda.default.join('.', versionArray);
};

const parseVersion = (input, regex) => {
  if (!input) {
    return _constants.BAD_VERSION;
  }

  const matches = input.match(regex);

  if (!matches || _ramda.default.isEmpty(matches)) {
    return _constants.BAD_VERSION;
  }

  const numericVersionNumbers = _ramda.default.filter(match => !Number.isNaN(match), matches);

  if (numericVersionNumbers.length > 0) {
    return normalizeVersion(numericVersionNumbers);
  }

  return matches[1];
};

exports.parseVersion = parseVersion;

const isAtleastGitVersion = gitInput => parseVersion(gitInput, _constants.regex.GIT) >= _constants.minimumVersions.GIT;

exports.isAtleastGitVersion = isAtleastGitVersion;

const isAtleastLfsVersion = lfsInput => parseVersion(lfsInput, _constants.regex.LFS) >= _constants.minimumVersions.LFS;

exports.isAtleastLfsVersion = isAtleastLfsVersion;

const setLfsFailed = response => {
  response.success = false;
  response.errno = _constants.BAD_VERSION;
  response.lfs_meets_version = false;
  response.lfs_exists = false;
  response.lfs_raw = null;
  response.lfs_version = null;
};

const setGitFailed = response => {
  response.success = false;
  response.errno = _constants.BAD_VERSION;
  response.git_meets_version = false;
  response.git_exists = false;
  response.git_raw = null;
  response.git_version = null;
};

const dependencyCheck = () => {
  const response = (0, _generateResponse.default)();
  return _lfsCommands.core.git('--version').then(({
    stdout,
    stderr
  }) => {
    if (stderr) {
      setGitFailed(response);
    } else {
      response.git_version = parseVersion(stdout, _constants.regex.GIT);
      response.git_meets_version = isAtleastGitVersion(stdout);
      response.git_exists = response.git_version !== _constants.BAD_VERSION;
      response.git_raw = stdout;
    }
  }).catch(() => {
    setGitFailed(response);
  }).then(() => _lfsCommands.core.git('lfs version')).then(({
    stdout,
    stderr
  }) => {
    if (stderr) {
      setLfsFailed(response);
    } else {
      response.lfs_version = parseVersion(stdout, _constants.regex.LFS);
      response.lfs_meets_version = isAtleastLfsVersion(stdout);
      response.lfs_exists = response.lfs_version !== _constants.BAD_VERSION;
      response.lfs_raw = stdout;
    }

    return response;
  }).catch(() => {
    setLfsFailed(response);
    return response;
  });
};

exports.dependencyCheck = dependencyCheck;