"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _child_process = require("child_process");

var _ramda = _interopRequireDefault(require("ramda"));

var _authService = require("./authService");

var _constants = require("../constants");

var _shellOptions = require("./shellOptions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const parseUrlFromErrorMessage = errorMessage => {
  let url = null;

  const matches = _constants.regex.CREDENTIALS_NOT_FOUND.exec(errorMessage);

  if (matches && matches.length > 1) {
    [url] = matches;
  }

  return url;
};

const spawnCommand = (command, opts, stdin = '') => new Promise((resolve, reject) => {
  const [cmd, ...args] = command.trim().split(' ');
  const childProcess = (0, _child_process.spawn)(cmd, args, _ramda.default.mergeDeepRight(opts, {
    stdio: 'pipe'
  }));
  const stdoutData = [];
  const stderrData = [];

  const makeDataAccumulator = accumulator => data => {
    accumulator.push(data);
  };

  childProcess.stdout.on('data', makeDataAccumulator(stdoutData));
  childProcess.stderr.on('data', makeDataAccumulator(stderrData));
  childProcess.on('error', () => {
    reject({
      status: null,
      stdout: '',
      stderr: ''
    });
  });
  childProcess.on('close', status => {
    const stdout = Buffer.concat(stdoutData);
    const stderr = Buffer.concat(stderrData);
    resolve({
      status,
      stdout,
      stderr
    });
  });
  childProcess.stdin.write(stdin);
  childProcess.stdin.end();
});

const spawn = async (command, stdin, opts = {}, credentialsCallback, repoPath = null) => {
  const resolvedStdin = stdin || '';

  const resolvedRepoPath = repoPath || _ramda.default.path('cwd', opts);

  const noAuthResult = await spawnCommand(command, (0, _shellOptions.combineShellOptions)(opts, {
    env: {
      GIT_TERMINAL_PROMPT: 0
    }
  }), resolvedStdin);

  if (noAuthResult.status === 0) {
    // then we're done, return the data
    return {
      stdout: noAuthResult.stdout
    };
  }

  const errorMessage = noAuthResult.stderr.toString();

  if (!_constants.regex.CREDENTIALS_ERROR.test(errorMessage)) {
    throw new Error(errorMessage);
  }

  await (0, _authService.ensureAuthServer)();
  const url = parseUrlFromErrorMessage(errorMessage);
  const credRequestId = (0, _authService.createCredRequestId)(resolvedRepoPath);

  const tryCredentialsUntilCanceled = async () => {
    const {
      username,
      password
    } = await credentialsCallback({
      type: 'CREDS_REQUESTED',
      credRequestId,
      repoPath: resolvedRepoPath,
      url
    });
    (0, _authService.storeUsernameAndPassword)(credRequestId, username, password);

    try {
      const authResult = await spawnCommand(command, (0, _shellOptions.combineShellOptions)(opts, {
        env: {
          GIT_TERMINAL_PROMPT: 0,
          GIT_ASKPASS: (0, _authService.getGitAskPassPath)(),
          NODEGIT_LFS_ASKPASS_STATE: credRequestId,
          NODEGIT_LFS_ASKPASS_PORT: (0, _authService.getAuthServerPort)(),
          NODEGIT_LFS_ASKPASS_PATH: (0, _authService.getGitAskPassClientPath)(),
          NODEGIT_LFS_NODE_PATH: (0, _authService.getNodeBinaryPath)()
        }
      }), resolvedStdin);

      if (authResult.status === 0) {
        await credentialsCallback({
          type: 'CREDS_SUCCEEDED',
          credRequestId,
          repoPath: resolvedRepoPath,
          verifiedCredentials: {
            username,
            password
          },
          url
        });
        (0, _authService.clearUsernameAndPassword)(credRequestId);
        return {
          stdout: authResult.stdout
        };
      }

      const stderr = authResult.stderr.toString();

      if (_constants.regex.CREDENTIALS_ERROR.test(stderr)) {
        const authError = new Error('Auth error');
        authError.isAuthError = true;
        throw authError;
      }

      throw new Error(stderr);
    } catch (e) {
      if (e.isAuthError) {
        (0, _authService.clearUsernameAndPassword)(credRequestId);
        await credentialsCallback({
          type: 'CREDS_FAILED',
          credRequestId,
          repoPath: resolvedRepoPath,
          url
        });
        return tryCredentialsUntilCanceled();
      }

      throw e;
    }
  };

  try {
    return await tryCredentialsUntilCanceled();
  } catch (e) {
    await credentialsCallback({
      type: 'CREDS_SPAWN_FAILED',
      error: e,
      credRequestId,
      repoPath
    });
    throw e;
  }
};

var _default = spawn;
exports.default = _default;